from plumbum.cmd import exportfs
from walt.common.tools import do, succeeds

IMAGE_EXPORT_PATTERN = """\
%(image_mountpoint)s %(walt_subnet)s(fsid=%(fsid)s,ro,sync,no_root_squash,no_subtree_check)
"""
PERSISTENT_EXPORT_PATTERN = """\
%(persist_mountpoint)s %(walt_subnet)s(rw,sync,no_root_squash,no_subtree_check)
"""

def generate_exports_file(root_paths, persist_paths, subnet):
    with open('/etc/exports', 'w') as f:
        f.write("# WALT NFS exports: this file is automatically generated.\n")
        f.write("# Root filesystem images\n")
        for root_path, fsid in root_paths:
            f.write(IMAGE_EXPORT_PATTERN % dict(
                    image_mountpoint=root_path,
                    walt_subnet=subnet,
                    fsid=fsid))
        f.write("# Persistent node directories\n")
        for persist_path in persist_paths:
            f.write(PERSISTENT_EXPORT_PATTERN % dict(
                    persist_mountpoint=persist_path,
                    walt_subnet=subnet))

def update_exports(root_paths, persist_paths, subnet):
    generate_exports_file(root_paths, persist_paths, subnet)
    exportfs('-r')
    ensure_nfsd_is_running()

def ensure_nfsd_is_running():
    if not succeeds('pidof nfsd >/dev/null'):
        do('service nfs-kernel-server restart')
